@@ -14,9 +14,9 @@
 #' to be expanded. In case of vector / matrix it is recommended to provide
 #' \code{ts} object, so the frequency of the data is taken into account.
 #' @param functions Vector of names for functions used.
-#' @param quiet If \code{quiet=FALSE}, then the progress is printed out.
+#' @param silent If \code{silent=FALSE}, then the progress is printed out.
 #' Otherwise the function won't print anything in the console.
-#' @param ... This is temporary and is needed in order to capture "quiet"
+#' @param ... This is temporary and is needed in order to capture "silent"
 #' parameter if it is provided.
 #'
 #' @return \code{ts} matrix with the transformed and the original variables
@@ -35,10 +35,7 @@
 #'
 #' @export xregTransformer
 
-xregTransformer <- function(xreg, functions=c("log", "exp", "inv", "sqrt", "square"), quiet=TRUE, ...){
-
-    #### This is temporary and needs to be removed at some point! ####
-    quiet[] <- depricator(quiet, list(...));
+xregTransformer <- function(xreg, functions=c("log", "exp", "inv", "sqrt", "square"), silent=TRUE, ...){
 
     # Check and prepare functions
     if(any(!(functions %in% c("log", "exp", "inv", "sqrt", "square")))){
@@ -51,7 +48,7 @@ xregTransformer <- function(xreg, functions=c("log", "exp", "inv", "sqrt", "squa
              call.=FALSE);
     }
 
-    if(!quiet){
+    if(!silent){
         cat("Preparing matrices...    ");
     }
 
@@ -86,7 +83,7 @@ xregTransformer <- function(xreg, functions=c("log", "exp", "inv", "sqrt", "squa
         xregNew <- ts(xregNew,start=xregStart,frequency=xregFrequency);
 
         for(i in 1:nExovars){
-            if(!quiet){
+            if(!silent){
                 if(i==1){
                     cat("\b");
                 }
@@ -113,7 +110,7 @@ xregTransformer <- function(xreg, functions=c("log", "exp", "inv", "sqrt", "squa
             }
             colnames(xregNew)[((i-1)*(nFunctions+1)+1):(i*(nFunctions+1))] <- c(xregNames[i],paste(functions,xregNames[i],sep="_"));
         }
-        if(!quiet){
+        if(!silent){
             cat(paste0(rep("\b",4),collapse=""));
             cat(" Done! \n");
         }
